@@ -5,10 +5,9 @@ use mutex::Mutex;
 
 use super::{
     input, output, BasicDisplay, Connection, Display, DisplayBase, PendingItem, RequestInfo,
-    EXT_KEY_SIZE,
+    StaticSetup, EXT_KEY_SIZE,
 };
 use crate::{
-    auto::xproto::Setup,
     event::Event,
     xid::{AtomicXidGenerator, XID},
 };
@@ -65,7 +64,8 @@ use spinning_top::Spinlock;
 ///
 /// Usage of `SyncDisplay` from a mutable reference is not recommended as, due to the synchronous primitives
 /// involved, mutable usage provides only marginal speedup as compared to immutable usage. If mutable usage is
-/// necessary, consider using the `BasicDisplay` structure instead, as it is thread-safe.
+/// necessary, consider using the `BasicDisplay` structure instead, as it is thread-safe and doesn't require
+/// any locking.
 #[derive(Debug)]
 pub struct SyncDisplay<Conn> {
     // the connection to the server
@@ -74,7 +74,7 @@ pub struct SyncDisplay<Conn> {
     io_lock: Mutex,
 
     // setup from the server
-    setup: Setup,
+    setup: StaticSetup,
 
     // xid generator
     xid: AtomicXidGenerator,
@@ -167,7 +167,7 @@ impl<Conn> From<BasicDisplay<Conn>> for SyncDisplay<Conn> {
 
 impl<Conn> DisplayBase for SyncDisplay<Conn> {
     #[inline]
-    fn setup(&self) -> &Setup {
+    fn setup(&self) -> &StaticSetup {
         &self.setup
     }
 
@@ -395,7 +395,7 @@ impl<Conn: AsyncConnection + Unpin> AsyncDisplay for SyncDisplay<Conn> {
 
 impl<'a, Conn> DisplayBase for &'a SyncDisplay<Conn> {
     #[inline]
-    fn setup(&self) -> &Setup {
+    fn setup(&self) -> &StaticSetup {
         &self.setup
     }
 
